package net.knarcraft.dynmapcitizens;

import net.knarcraft.dynmapcitizens.handler.VaultHandler;
import net.knarcraft.dynmapcitizens.handler.trait.BlacksmithHandler;
import net.knarcraft.dynmapcitizens.handler.trait.CitizensTraitHandler;
import net.knarcraft.dynmapcitizens.handler.trait.MinstrelHandler;
import net.knarcraft.dynmapcitizens.handler.trait.SentinelHandler;
import net.knarcraft.dynmapcitizens.handler.trait.quests.QuestsHandler;
import net.knarcraft.dynmapcitizens.property.Icon;
import org.bukkit.Bukkit;
import org.bukkit.plugin.Plugin;
import org.bukkit.plugin.PluginManager;
import org.bukkit.plugin.java.JavaPlugin;
import org.dynmap.DynmapAPI;
import org.dynmap.markers.MarkerAPI;
import org.dynmap.markers.MarkerIcon;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.logging.Level;

@SuppressWarnings("unused")
public final class DynmapCitizens extends JavaPlugin {

    private static DynmapCitizens instance;
    private DynmapAPI dynmapAPI;
    private Map<Icon, MarkerIcon> markerIcons;
    private VaultHandler vaultHandler;

    @Override
    public void onEnable() {
        instance = this;
        //Initialize quest and dynmap APIs
        PluginManager pluginManager = Bukkit.getPluginManager();
        Plugin dynmapPlugin = pluginManager.getPlugin("dynmap");
        if (!(dynmapPlugin instanceof DynmapAPI dynmapAPI) || dynmapAPI.getMarkerAPI() == null) {
            this.getLogger().log(Level.SEVERE, "Could not initialize Dynmap");
            this.onDisable();
            return;
        }
        this.dynmapAPI = dynmapAPI;

        //Sets all icons used for displaying NPC locations
        loadMarkerIcons();

        //Setup all trait handlers
        List<CitizensTraitHandler> handlers = new ArrayList<>();
        handlers.add(new BlacksmithHandler());
        handlers.add(new QuestsHandler());
        handlers.add(new SentinelHandler());
        handlers.add(new MinstrelHandler());
        for (CitizensTraitHandler handler : handlers) {
            handler.initialize();
            handler.updateMarkers();
        }

        //Schedule handlers to run periodically
        Bukkit.getScheduler().runTaskTimer(this, () -> {
            for (CitizensTraitHandler handler : handlers) {
                if (handler.isEnabled()) {
                    handler.updateMarkers();
                }
            }
        }, 10 * 20, 300 * 20);

        vaultHandler = new VaultHandler(this.getServer().getServicesManager());
    }

    @Override
    public void onDisable() {

    }

    /**
     * Gets the Vault handler to use for anything Vault-related
     *
     * @return <p>The Vault handler</p>
     */
    public VaultHandler getVaultHandler() {
        return vaultHandler;
    }

    /**
     * Gets a reference to the Dynmap API
     *
     * @return <p>A reference to the Dynmap API</p>
     */
    public DynmapAPI getDynmapAPI() {
        return this.dynmapAPI;
    }

    /**
     * Gets a map of loaded marker icons
     *
     * @return <p>All marker icons</p>
     */
    public Map<Icon, MarkerIcon> getMarkerIcons() {
        return new HashMap<>(this.markerIcons);
    }

    /**
     * Gets an instance of this plugin
     *
     * @return <p>An instance of this plugin</p>
     */
    public static DynmapCitizens getInstance() {
        return instance;
    }

    /**
     * Loads necessary marker icons, and updates icons for the dynmap drawer
     */
    private void loadMarkerIcons() {
        //TODO: Make every icon configurable
        MarkerAPI markerAPI = dynmapAPI.getMarkerAPI();
        Map<Icon, MarkerIcon> markerIcons = new HashMap<>();
        markerIcons.put(Icon.QUEST_GIVER, markerAPI.getMarkerIcon("exclamation"));
        markerIcons.put(Icon.QUEST_DELIVER, markerAPI.getMarkerIcon("basket"));
        markerIcons.put(Icon.QUEST_KILL, markerAPI.getMarkerIcon("skull"));
        markerIcons.put(Icon.QUEST_INTERACT, markerAPI.getMarkerIcon("comment"));
        markerIcons.put(Icon.BLACKSMITH, markerAPI.getMarkerIcon("hammer"));
        markerIcons.put(Icon.SENTINEL, markerAPI.getMarkerIcon("shield"));
        markerIcons.put(Icon.QUEST_CHAIN, markerAPI.getMarkerIcon("caution"));
        markerIcons.put(Icon.MINSTREL, markerAPI.getMarkerIcon("theater"));
        this.markerIcons = markerIcons;
    }

}
