package net.knarcraft.dynmapcitizens.handler.trait.quests;

import me.blackvein.quests.quests.IQuest;
import me.blackvein.quests.quests.Requirements;
import net.knarcraft.dynmapcitizens.util.QuestsHelper;
import org.bukkit.inventory.ItemStack;

import java.util.List;
import java.util.Map;

/**
 * A class to generate a string containing all information about a quest's requirements
 */
public class QuestRequirementsInfoGenerator {

    private final IQuest quest;

    /**
     * Instantiates a new quest requirement info generator
     *
     * @param quest <p>The quest to generate information about</p>
     */
    public QuestRequirementsInfoGenerator(IQuest quest) {
        this.quest = quest;
    }

    /**
     * Gets information about all requirements for the given quest
     *
     * @return <p>Information about the quest's requirements</p>
     */
    public String getQuestRequirementsInfo() {
        Requirements requirements = quest.getRequirements();
        StringBuilder requirementInfo = new StringBuilder();
        if (!requirements.hasRequirement()) {
            return requirementInfo.toString();
        }

        requirementInfo.append("<b>Requirements: </b><ul>");

        if (requirements.getQuestPoints() > 0) {
            requirementInfo.append("<li>").append(requirements.getQuestPoints()).append(" quest points</li>");
        }

        if (requirements.getExp() > 0) {
            requirementInfo.append("<li>").append(requirements.getExp()).append(" exp</li>");
        }

        if (!requirements.getBlockQuests().isEmpty()) {
            requirementInfo.append("<li>Blocked by quests:<ul>");
            for (IQuest blockQuest : requirements.getBlockQuests()) {
                requirementInfo.append("<li>").append(blockQuest.getName()).append("</li>");
            }
            requirementInfo.append("</ul></li>");
        }

        if (!requirements.getNeededQuests().isEmpty()) {
            requirementInfo.append("<li>Required quests:<ul>");
            for (IQuest neededQuest : requirements.getNeededQuests()) {
                requirementInfo.append("<li>").append(neededQuest.getName()).append("</li>");
            }
            requirementInfo.append("</ul></li>");
        }

        if (!requirements.getItems().isEmpty()) {
            requirementInfo.append("<li>Required items:<ul>");
            for (ItemStack item : requirements.getItems()) {
                requirementInfo.append("<li>").append(QuestsHelper.getUpperCasedItemStackString(item)).append("</li>");
            }
            requirementInfo.append("</ul></li>");
        }

        if (!requirements.getMcmmoSkills().isEmpty()) {
            List<String> skills = requirements.getMcmmoSkills();
            List<Integer> amounts = requirements.getMcmmoAmounts();
            for (int i = 0; i < skills.size(); i++) {
                requirementInfo.append("<li>Requires mcMMO skill ").append(skills.get(i)).append(" at level ");
                requirementInfo.append(amounts.get(i)).append("</li>");
            }
        }

        if (!requirements.getPermissions().isEmpty()) {
            requirementInfo.append("<li>Required permissions:<ul>");
            for (String permission : requirements.getPermissions()) {
                requirementInfo.append("<li>").append(permission).append("</li>");
            }
            requirementInfo.append("</ul></li>");
        }

        Map<String, Map<String, Object>> customRequirementPlugins = requirements.getCustomRequirements();
        for (String plugin : customRequirementPlugins.keySet()) {
            requirementInfo.append("<li>").append(plugin).append(":<ul>");
            //Note: The format of custom requirements is kind of weird. First, you have the key for which plugin the 
            // requirement belongs to. Getting the value of the key gives another map. The map contains as key, the type
            // of value, like "Skill Amount" or "Skill Type". The value is the actual value of whatever it is.
            Map<String, Object> customRequirementEntry = customRequirementPlugins.get(plugin);
            for (String requirementDescription : customRequirementEntry.keySet()) {
                requirementInfo.append("<li>").append(requirementDescription).append(" ");
                requirementInfo.append(customRequirementEntry.get(requirementDescription)).append("</li>");
            }
            requirementInfo.append("</ul></li>");
        }

        requirementInfo.append("</ul>");
        return requirementInfo.toString();
    }

}
